<?php

namespace Drupal\sfc;

/**
 * A base class for single file components that pre-compile assets.
 *
 * If you extend this class, you can run the "sfc:watch" or "sfc:build"
 * commands to write your assets to the <module root>/sfc/components/<name>/src
 * directory. From there you can compile these assets yourself to the
 * [...]/dist directory using your tooling of choice.
 *
 * You'll probably also want to .gitignore the generated src directories,
 * unless you want to keep it around to keep map files working.
 *
 * Unlike normal single file components, assets will not be written when
 * requested.
 */
class CompiledComponentBase extends ComponentBase implements CompiledComponentInterface {

  /**
   * An optional Saas string. Useful because the output file ends in ".scss".
   */
  const SASS = NULL;

  /**
   * {@inheritdoc}
   */
  public function getLibrary() {
    $library = parent::getLibrary();
    if ($this->hasSaas()) {
      $directory = $this->getAssetPath();
      $name = $this->getAssetFilename();
      $library['css']['theme'][file_url_transform_relative(file_create_url("$directory/$name.css"))] = [];
    }
    return $library;
  }

  /**
   * {@inheritdoc}
   */
  public function writeSource() {
    $directory = $this->appRoot . '/' . str_replace('dist', 'src', $this->getAssetPath());
    $name = $this->getAssetFilename();
    $this->doWriteAssets($directory, $name);
    if ($this::hasSaas()) {
      file_put_contents("$directory/$name.scss", $this->getSaas());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function shouldWriteSource() {
    if ($this->hasAssets()) {
      $directory = $this->appRoot . '/' . str_replace('dist', 'src', $this->getAssetPath());
      $name = $this->getAssetFilename();
      return $this->areAssetsOutdated($directory, $name);
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  protected function areAssetsOutdated($directory, $name) {
    $obj = new \ReflectionClass($this);
    $filename = $obj->getFileName();
    if ($this->hasSaas()) {
      if (!file_exists("$directory/$name.scss")) {
        return TRUE;
      }
      if (filemtime($filename) > filemtime("$directory/$name.scss")) {
        return TRUE;
      }
    }
    return parent::areAssetsOutdated($directory, $name);
  }

  /**
   * {@inheritdoc}
   */
  protected function hasAssets() {
    return parent::hasAssets() || $this->hasSaas();
  }

  /**
   * {@inheritdoc}
   */
  protected function getAssetPath() {
    $directory = $this->getProjectPath();
    $name = preg_replace('/[^A-Za-z0-9_]/', '_', $this->getId());
    return $directory . "/assets/components/$name/dist";
  }

  /**
   * {@inheritdoc}
   */
  public function writeAssets() {}

  /**
   * {@inheritdoc}
   */
  public function shouldWriteAssets() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  protected function getCss() {
    // Do not prefix CSS with the module path.
    return $this::CSS;
  }

  /**
   * Gets Sass for this component.
   *
   * @return string|null
   *   The Sass for this component, or NULL if none is defined.
   */
  protected function getSaas() {
    return $this::SASS;
  }

  /**
   * Determines whether or not this component defines Sass.
   *
   * @return bool
   *   Whether or not this component defines Sass.
   */
  protected function hasSaas() {
    return (bool) $this::SASS;
  }

}
