<?php

namespace Drupal\Tests\sfc\Unit;

use Drupal\Core\File\FileSystemInterface;
use Drupal\sfc\CompiledComponentBase;
use Drupal\Tests\UnitTestCase;
use org\bovigo\vfs\vfsStream;

/**
 * Tests methods provided by the base component.
 *
 * @coversDefaultClass \Drupal\sfc\CompiledComponentBase
 *
 * @group sfc
 */
class CompiledComponentBaseTest extends UnitTestCase {

  /**
   * Tests the ::getLibrary method.
   */
  public function testGetLibrary() {
    $file_system = $this->createMock(FileSystemInterface::class);
    $component = new TestCompiledComponent([], 'test_component', [
      'provider' => 'sfc',
    ], FALSE, 'vfs:/', $file_system);
    $this->assertEquals([
      'css' => [
        'theme' => [
          realpath(__DIR__ . '/../../..') . '/assets/components/test_component/dist/test_component.css' => [],
        ],
      ],
    ], $component->getLibrary());
  }

  /**
   * Tests the ::writeSource method.
   */
  public function testWriteSource() {
    vfsStream::setup('sfc');
    $scss_file = 'vfs://sfc/components/test_component/src/test_component.scss';
    $css_file = 'vfs://sfc/components/test_component/src/test_component.css';
    mkdir('vfs://sfc/components/test_component/src', 0777, TRUE);
    $file_system = $this->createMock(FileSystemInterface::class);
    $component = new WriteTestCompiledComponent([], 'test_component', [], FALSE, 'vfs:/', $file_system);
    $this->assertFileNotExists($scss_file);
    $component->writeSource();
    $this->assertFileExists($scss_file);
    $this->assertFileExists($css_file);
    // Compiled CSS paths should not be prefixed.
    $this->assertStringEqualsFile($css_file, '.foo { bar: url(./baz); };');
  }

  /**
   * Tests the ::shouldWriteAssets method.
   */
  public function testShouldWriteAssets() {
    $file_system = $this->createMock(FileSystemInterface::class);
    $component = new WriteTestCompiledComponent([], 'test_component', [], FALSE, 'vfs:/', $file_system);
    $this->assertFalse($component->shouldWriteAssets());
  }

  /**
   * Tests the ::shouldWriteSource method.
   */
  public function testShouldWriteSource() {
    $scss_file = 'vfs://sfc/components/test_component/src/test_component.scss';
    $css_file = 'vfs://sfc/components/test_component/src/test_component.css';
    mkdir('vfs://sfc/components/test_component/src', 0777, TRUE);
    $file_system = $this->createMock(FileSystemInterface::class);
    $component = new WriteTestCompiledComponent([], 'test_component', [], FALSE, 'vfs:/', $file_system);
    // If the scss file doesn't exist.
    $this->assertTrue($component->shouldWriteSource());
    // If the files exist.
    touch($scss_file);
    touch($css_file);
    $this->assertFalse($component->shouldWriteSource());
    // If the scss file is out of date.
    touch($scss_file, 0);
    $this->assertTrue($component->shouldWriteSource());
    $component = new EmptyTestCompiledComponent([], 'test_component', [], FALSE, 'vfs:/', $file_system);
    // If no assets are defined.
    $this->assertFalse($component->shouldWriteSource());
  }

  /**
   * {@inheritdoc}
   *
   * @codeCoverageIgnore
   */
  protected function tearDown() {
    vfsStream::setup('sfc');
    if (is_file('vfs://sfc/components/test_component/src/test_component.scss')) {
      unlink('vfs://sfc/components/test_component/src/test_component.scss');
    }
    if (is_dir('vfs://sfc/components/test_component')) {
      rmdir('vfs://sfc/components/test_component/src');
      rmdir('vfs://sfc/components/test_component');
      rmdir('vfs://sfc/components');
    }
  }

}

// phpcs:disable
// @codeCoverageIgnoreStart

class EmptyTestCompiledComponent extends CompiledComponentBase {}

class TestCompiledComponent extends CompiledComponentBase {

  const SASS = 'foo';

}

class WriteTestCompiledComponent extends CompiledComponentBase {

  const SASS = 'foo';

  const CSS = '.foo { bar: url(./baz); };';

  protected function getAssetPath() {
    return 'sfc/components/test_component/src';
  }

}

namespace Drupal\sfc;

if (!function_exists('Drupal\sfc\file_url_transform_relative')) {
  function file_url_transform_relative($file_url) {
    return $file_url;
  }
}

if (!function_exists('Drupal\sfc\file_create_url')) {
  function file_create_url($uri) {
    return $uri;
  }
}

// phpcs:enable
// @codeCoverageIgnoreEnd
